@@ -10,7 +10,7 @@ from loguru import logger
 
 from .exceptions import ConnectionError, QueryError, TableExistsError
 from .helpers import generate_field_metadata
-from .models import FetchParams
+from .models import FetchParams, FieldsParams
 
 
 def attach_motherduck(func):
@@ -41,7 +41,7 @@ class HdDB:
     ) -> duckdb.DuckDBPyConnection:
         return self.conn.execute(query, parameters)
 
-    def create_database(self, dataframes: List[pd.DataFrame], names: List[str]):
+    def create_database(self, org: str, db: str, dataframes: List[pd.DataFrame], names: List[str]):
         """
         Create in-memory database and create tables from a list of dataframes.
 
@@ -86,9 +86,25 @@ class HdDB:
                 all_metadata.extend(metadata)
 
             self.create_hd_tables()
+            self.create_hd_database(org, db, len(dataframes))
             self.create_hd_fields(all_metadata)
         except duckdb.Error as e:
             raise QueryError(f"Error executing query: {e}")
+        
+    def create_hd_database(self, org: str, db: str, tables: int):
+        try:
+            self.execute("BEGIN TRANSACTION;")
+            
+            create_query = "CREATE TABLE hd_database (id VARCHAR, username VARCHAR, slug VARCHAR, db_created_at TIMESTAMP DEFAULT current_timestamp, db_updated_at TIMESTAMP DEFAULT current_timestamp, db_n_tables INTEGER);"
+            self.execute(create_query)
+            
+            insert_query = "INSERT INTO hd_database (id, username, slug, db_n_tables) VALUES (?, ?, ?, ?);"
+            self.execute(insert_query, [f"{org}__{db}", org, db, tables])
+            
+            self.execute("COMMIT;")
+        except duckdb.Error as e:
+            self.execute("ROLLBACK;")
+            raise QueryError(f"Error executing query: {e}")
 
     # TODO: map duckdb data types to datasketch types
     def create_hd_fields(self, metadata: List[Dict[str, str]]):
@@ -206,7 +222,12 @@ class HdDB:
             start_row = params.get("start_row", 0)
             page_size = end_row - start_row
 
-            query = f'SELECT * FROM "{org}__{db}"."{tbl}" LIMIT {page_size} OFFSET {start_row}'
+            query = f'SELECT * FROM "{org}__{db}"."{tbl}"'
+
+            if params.get("sort"):
+                query += f" ORDER BY {params.get('sort')}"
+
+            query += f" LIMIT {page_size} OFFSET {start_row}"
 
             data = self.execute(query).fetchdf()
 
@@ -686,7 +707,7 @@ class HdDB:
             raise QueryError(f"Error updating row in table {tbl}: {e}")
 
     @attach_motherduck
-    def get_fields(self, org: str, db: str, tbl: Optional[str] = None) -> list:
+    def get_fields(self, org: str, db: str, tbl: Optional[str] = None, params: Optional[FieldsParams] = None) -> list:
         """
         Retrieve fields from the hd_fields table for a given table.
 
@@ -701,8 +722,34 @@ class HdDB:
             if tbl is not None:
                 query += " WHERE tbl = ?"
             result = self.execute(query, [tbl] if tbl is not None else None).fetchdf()
+            data = json.loads(result.to_json(orient="records"))
 
-            return {"data": json.loads(result.to_json(orient="records"))}
+            if params.with_categories:
+                for row in data:
+                    column_type, column_id = row.get('type'), row.get('id')
+                    if column_type == 'Cat':
+                        query = f'SELECT DISTINCT({column_id}) FROM "{org}__{db}".{tbl}'
+                        categories = self.execute(query).fetchall()
+                        row['categories'] = [c[0] for c in categories]
+
+
+            return {"data": data}
         except duckdb.Error as e:
             logger.error(f"Error fetching fields from hd_fields: {e}")
             raise QueryError(f"Error fetching fields from hd_fields: {e}")
+
+    @attach_motherduck
+    def get_metadata(self, org: str, db: str, tbl: str):
+        try:
+            query = f'SELECT * FROM "{org}__{db}".hd_tables WHERE id = ?'
+            result = self.execute(query, [tbl]).fetchdf()
+            data = result.to_dict(orient="records")
+            nrow, ncol, tbl_name = (
+                data[0]["nrow"],
+                data[0]["ncol"],
+                data[0]["id"],
+            )
+            return {"nrow": nrow, "ncol": ncol, "tbl_name": tbl_name}
+        except duckdb.Error as e:
+            logger.error(f"Error fetching metadata from hd_fields: {e}")
+            raise QueryError(f"Error fetching metadata from hd_fields: {e}")
